package download

import (
	"context"
	"downloader/internal/app/api/common/responses"
	"downloader/internal/core/entities"
	"errors"
	"github.com/go-chi/render"
	"net/http"
)

type requestDownloadRequest struct {
	Link string `json:"link"`
}

type requestDownloadResponse struct {
	*entities.Download
}

func (_ requestDownloadResponse) Render(_ http.ResponseWriter, _ *http.Request) error {
	return nil
}

func (dr *requestDownloadRequest) Bind(r *http.Request) error {
	if dr.Link == "" {
		return errors.New("missing required download request field")
	}

	return nil
}

func (a *api) requestDownload(w http.ResponseWriter, r *http.Request) {
	ctx := r.Context()
	data := &requestDownloadRequest{}
	if err := render.Bind(r, data); err != nil {
		_ = render.Render(w, r, responses.ErrInvalidRequest(err))
		return
	}

	download, err := a.drService.Schedule(ctx, data.Link)
	if err != nil {
		_ = render.Render(w, r, responses.ErrInvalidRequest(err))
		return
	}

	render.Status(r, http.StatusAccepted)
	_ = render.Render(w, r, newRequestDownloadResponse(download))
}

func (a *api) getDownloads(writer http.ResponseWriter, request *http.Request) {
	ctx := request.Context()

	if done := request.URL.Query().Get("done"); done == "true" {
		_ = a.getDoneDownloads(writer, request, ctx)
		return
	}

	active := request.URL.Query().Get("active") == "true"
	downloads, err := a.drService.GetAll(ctx, active)
	if err != nil {
		_ = render.Render(writer, request, responses.ErrInvalidRequest(err))
		return
	}

	if err = render.RenderList(writer, request, newDownloadsResponse(downloads)); err != nil {
		_ = render.Render(writer, request, responses.ErrInvalidRequest(err))
		return
	}
}

func (a *api) getDoneDownloads(writer http.ResponseWriter, request *http.Request, ctx context.Context) bool {
	downloads, err := a.drService.GetDone(ctx)
	if err != nil {
		_ = render.Render(writer, request, responses.ErrInvalidRequest(err))
		return true
	}

	if err = render.RenderList(writer, request, newDownloadsResponse(downloads)); err != nil {
		_ = render.Render(writer, request, responses.ErrInvalidRequest(err))
		return true
	}
	return false
}

func (a *api) getDownloadById(w http.ResponseWriter, r *http.Request) {
	ctx := r.Context()
	downloadId := ctx.Value("downloadId").(string)

	download, err := a.drService.Get(ctx, downloadId)
	if err != nil {
		_ = render.Render(w, r, responses.ErrNotFound())
		return
	}

	if err := render.Render(w, r, newRequestDownloadResponse(download)); err != nil {
		_ = render.Render(w, r, responses.ErrInvalidRequest(err))
		return
	}
}

func newRequestDownloadResponse(download *entities.Download) *requestDownloadResponse {
	return &requestDownloadResponse{Download: download}
}

func newDownloadsResponse(downloads []*entities.Download) []render.Renderer {
	var list []render.Renderer
	for _, download := range downloads {
		list = append(list, newRequestDownloadResponse(download))
	}

	if len(list) == 0 {
		return []render.Renderer{}
	}

	return list
}
