package sql

import (
	"context"
	"downloader/internal/core/entities"
	"downloader/internal/core/ports/download_request"
	"github.com/uptrace/bun"
	"go.uber.org/zap"
)

type repository struct {
	db     *bun.DB
	logger *zap.SugaredLogger
}

func NewDownloadRequestSqlRepository(db *bun.DB, logger *zap.SugaredLogger) download_request.Repository {
	return &repository{
		db:     db,
		logger: logger,
	}
}

func (r repository) Create(ctx context.Context, download *entities.Download) (*entities.Download, error) {
	insertDownload := &Download{
		ID:     download.ID,
		Status: download.Status,
		Link:   download.Link,
	}

	_, err := r.db.NewInsert().
		Model(insertDownload).
		Returning("*").
		Exec(ctx)
	if err != nil {
		return nil, err
	}

	return &entities.Download{
		ID:     insertDownload.ID,
		Status: insertDownload.Status,
		Link:   insertDownload.Link,
	}, nil
}

func (r repository) GetById(ctx context.Context, id string) (*entities.Download, error) {
	download := &Download{
		ID: id,
	}

	err := r.db.NewSelect().
		Model(download).
		WherePK().
		Scan(ctx)
	if err != nil {
		return nil, err
	}

	return &entities.Download{
		ID:     download.ID,
		Status: download.Status,
		Link:   download.Link,
	}, nil
}

func (r repository) Update(ctx context.Context, download *entities.Download) error {
	updateDownload := &Download{
		ID:     download.ID,
		Status: download.Status,
		Link:   download.Link,
	}

	_, err := r.db.NewUpdate().
		Model(updateDownload).
		ExcludeColumn("created_at").
		WherePK().
		Exec(ctx)

	return err
}

func (r repository) Get(ctx context.Context, active bool) ([]*entities.Download, error) {
	var downloads []Download
	query := r.db.NewSelect().
		Model(&downloads).
		Column("id", "status", "link")

	var err error
	if active {
		err = query.
			Where("status LIKE ? OR status = ?", "in-progress%", "scheduled").
			Order("created_at ASC").
			Scan(ctx)
	} else {
		err = query.
			Order("created_at ASC").
			Scan(ctx)
	}

	if err != nil {
		return nil, err
	}

	var responseDownloads []*entities.Download
	for _, download := range downloads {
		responseDownloads = append(responseDownloads, &entities.Download{
			ID:     download.ID,
			Status: download.Status,
			Link:   download.Link,
		})
	}

	return responseDownloads, nil
}

func (r repository) GetOldOrStuck(ctx context.Context) ([]*entities.Download, error) {

	var downloads []Download
	err := r.db.NewSelect().
		Model(&downloads).
		Column("id", "status", "link").
		Where("status LIKE ? OR status = ?", "in-progress%", "scheduled").
		Where("updated_at < now() - interval '1 minutes'").
		Limit(20).
		Order("created_at ASC").
		Scan(ctx)
	if err != nil {
		return nil, err
	}

	var responseDownloads []*entities.Download
	for _, download := range downloads {
		responseDownloads = append(responseDownloads, &entities.Download{
			ID:     download.ID,
			Status: download.Status,
			Link:   download.Link,
		})
	}
	if len(responseDownloads) == 0 {
		return []*entities.Download{}, nil
	}

	return responseDownloads, nil
}

func (r repository) BatchDelete(ctx context.Context, requests []*entities.Download) error {
	var downloads []*Download
	for _, request := range requests {
		downloads = append(downloads, &Download{ID: request.ID})
	}
	if len(downloads) == 0 {
		return nil
	}

	_, err := r.db.NewDelete().
		Model(&downloads).
		WherePK().
		Exec(ctx)
	if err != nil {
		return err
	}

	return nil
}

func (r repository) GetDone(ctx context.Context) ([]*entities.Download, error) {
	var downloads []Download
	err := r.db.NewSelect().
		Model(&downloads).
		Column("id", "status", "link").
		Where("status = ?", "done").
		Order("created_at ASC").
		Scan(ctx)

	if err != nil {
		return nil, err
	}

	var responseDownloads []*entities.Download
	for _, download := range downloads {
		responseDownloads = append(responseDownloads, &entities.Download{
			ID:     download.ID,
			Status: download.Status,
			Link:   download.Link,
		})
	}

	return responseDownloads, nil
}
